import logging
import argparse
import base64
from hashlib import sha256
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad

# 配置日志记录器
logging.basicConfig(level=logging.DEBUG, format='[%(asctime)s] [%(levelname)s] %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
logger = logging.getLogger()

def hash_message(message: str) -> str:
    """对消息进行哈希处理"""
    logger.info("开始哈希消息")
    hashed_bytes = sha256(message.encode()).digest()
    hashed_message = base64.b64encode(hashed_bytes).decode()
    logger.info("消息哈希成功")
    return hashed_message

def encrypt_message(message: str, key: str) -> str:
    """加密消息，使用 AES 对称加密"""
    logger.info("开始加密消息")
    key_hash = sha256(key.encode()).digest()[:16]
    cipher = AES.new(key_hash, AES.MODE_CBC)
    ct_bytes = cipher.encrypt(pad(message.encode(), AES.block_size))
    encrypted_message = base64.b64encode(cipher.iv + ct_bytes).decode()
    logger.info("消息加密成功")
    return encrypted_message

def decrypt_message(encrypted_message: str, key: str) -> str:
    """解密消息，使用 AES 对称加密"""
    logger.info("开始解密消息")
    key_hash = sha256(key.encode()).digest()[:16]
    encrypted_bytes = base64.b64decode(encrypted_message.encode())
    iv = encrypted_bytes[:AES.block_size]
    ct = encrypted_bytes[AES.block_size:]
    cipher = AES.new(key_hash, AES.MODE_CBC, iv)
    decrypted_bytes = unpad(cipher.decrypt(ct), AES.block_size)
    decrypted_message = decrypted_bytes.decode()
    logger.info("消息解密成功")
    return decrypted_message

def encrypt_file(file_path: str, key: str):
    """加密文件并保存为 .bmi 格式"""
    logger.info(f"开始加密文件: {file_path}")
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            original_message = file.read()
    except Exception as e:
        logger.error(f"读取文件失败: {e}")
        return
    
    encrypted_message = encrypt_message(original_message, key)
    encrypted_file_path = f"{file_path}.bmi"
    with open(encrypted_file_path, 'w', encoding='utf-8') as encrypted_file:
        encrypted_file.write(encrypted_message)
    logger.info(f"文件加密成功，保存为: {encrypted_file_path}")

def decrypt_file(encrypted_file_path: str, key: str):
    """解密文件"""
    logger.info(f"开始解密文件: {encrypted_file_path}")
    try:
        with open(encrypted_file_path, 'r', encoding='utf-8') as encrypted_file:
            encrypted_message = encrypted_file.read()
    except Exception as e:
        logger.error(f"读取加密文件失败: {e}")
        return
    
    decrypted_message = decrypt_message(encrypted_message, key)
    decrypted_file_path = encrypted_file_path.replace('.bmi', '')
    with open(decrypted_file_path, 'w', encoding='utf-8') as decrypted_file:
        decrypted_file.write(decrypted_message)
    logger.info(f"文件解密成功，保存为: {decrypted_file_path}")

def show_help():
    """显示详细的帮助信息"""
    help_message = """
ByMimi 加密工具使用说明：

命令行参数：
  -e, --encrypt <消息>      加密一段消息
  -d, --decrypt <消息>      解密一段消息
  -hs, --hash <消息>        计算消息的哈希值
  -k, --key <密钥>          用于加密或解密的密钥
  -f, --file <文件路径>     加密或解密文件
  -help                     显示帮助信息

示例用法：
  1. 加密消息：
     python script.py -e "你的消息" -k "你的密钥"

  2. 解密消息：
     python script.py -d "加密后的消息" -k "你的密钥"

  3. 计算哈希：
     python script.py -hs "你的消息"

  4. 加密文件：
     python script.py -e -f "文件路径" -k "你的密钥"

  5. 解密文件：
     python script.py -d -f "加密文件路径" -k "你的密钥"

请确保提供正确的密钥，以便加密和解密消息或文件。
"""
    print(help_message)

def main():
    parser = argparse.ArgumentParser(add_help=False)  # 禁用默认的帮助
    parser.add_argument('-e', '--encrypt', type=str, help="加密消息")
    parser.add_argument('-d', '--decrypt', type=str, help="解密消息")
    parser.add_argument('-hs', '--hash', type=str, help="哈希消息")
    parser.add_argument('-k', '--key', type=str, help="用于加密/解密的密钥")
    parser.add_argument('-f', '--file', type=str, help="加密或解密文件")
    parser.add_argument('-help', action='store_true', help="显示详细的帮助信息")

    args = parser.parse_args()

    if args.help:
        show_help()
    elif args.encrypt and args.key:
        try:
            if args.file:
                encrypt_file(args.file, args.key)
            else:
                encrypted_message = encrypt_message(args.encrypt, args.key)
                print(f"加密结果: {encrypted_message}")
        except Exception as e:
            logger.error(f"加密过程中发生错误: {e}")
    elif args.decrypt and args.key:
        try:
            if args.file:
                decrypt_file(args.file, args.key)
            else:
                decrypted_message = decrypt_message(args.decrypt, args.key)
                print(f"解密结果: {decrypted_message}")
        except Exception as e:
            logger.error(f"解密过程中发生错误: {e}")
    elif args.hash:
        try:
            hashed_message = hash_message(args.hash)
            print(f"哈希结果: {hashed_message}")
        except Exception as e:
            logger.error(f"哈希过程中发生错误: {e}")
    else:
        logger.error("请提供有效的参数：使用 -e 加密消息，-d 解密消息，或 -hs 哈希消息，并提供密钥（如适用）。使用 -help 获取详细的帮助信息。")

if __name__ == "__main__":
    main()